#!/usr/bin/python
# -*- coding: utf-8 -*-

### BEGIN LICENSE
# Copyright (C) 2013 ~ 2019 National University of Defense Technology(NUDT) & Kylin Ltd
#
# Authors:
# Kobe Lee    lixiang@kylinos.cn/xiangli@ubuntukylin.com/kobe24_lixiang@126.com
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os, sys
import re
import binascii
import threading
import platform
import signal
import subprocess
import dbus
import dbus.service
import dbus.mainloop.glib
import math
from gi.repository import GLib, GObject
#import apt
import gettext
#locale_path = '/usr/share/locale/'

gettext.textdomain('kylin-computer-viewer')
def _(s):
    return gettext.gettext(s)

CPU_MAX_FREQ = "/sys/devices/system/cpu/cpu0/cpufreq/scaling_max_freq"
MEMORY = "/sys/phytium1500a_info"
RELEASEPATH = "/etc/lsb-release"
OSPATH = "/etc/os-release"

KILOBYTE_FACTOR = 1024.0
MEGABYTE_FACTOR = (1024.0 * 1024.0)
GIGABYTE_FACTOR = (1024.0 * 1024.0 * 1024.0)
TERABYTE_FACTOR = (1024.0 * 1024.0 * 1024.0 * 1024.0)

MEM_KILOBYTE_FACTOR = 1024.0
MEM_MEGABYTE_FACTOR = (1024.0 * 1024.0)
MEM_GIGABYTE_FACTOR = (1024.0 * 1024.0 * 1024.0)
MEM_TERABYTE_FACTOR = (1024.0 * 1024.0 * 1024.0 * 1024.0)

def get_human_read_capacity_size(size):
    '''
    将容量大小转换为用户可以理解的值和单位
    '''
    size_str = ""
    displayed_size = 0.0
    unit = "KiB"

    if size < MEGABYTE_FACTOR:
        displayed_size = float(size/KILOBYTE_FACTOR)
        unit = "KiB"
    elif size < GIGABYTE_FACTOR:
        displayed_size = float(size/MEGABYTE_FACTOR)
        unit = "MiB"
    elif size < TERABYTE_FACTOR:
        displayed_size = float(size/GIGABYTE_FACTOR)
        unit = "GiB"
    else:
        displayed_size = float(size/TERABYTE_FACTOR)
        unit = "TiB"

    str_list = [str("%.1f" % displayed_size), unit]
    size_str = " ".join(str_list)
    return size_str

def get_mem_human_read_capacity_size(size):
    '''
    将容量大小转换为用户可以理解的值和单位
    '''
    size_str = ""
    displayed_size = 0.0
    unit = "KB"

    if size < MEM_MEGABYTE_FACTOR:
        displayed_size = float(size/MEM_KILOBYTE_FACTOR)
        unit = "KB"
    elif size < MEM_GIGABYTE_FACTOR:
        displayed_size = float(size/MEM_MEGABYTE_FACTOR)
        unit = "MB"
    elif size < MEM_TERABYTE_FACTOR:
        displayed_size = float(size/MEM_GIGABYTE_FACTOR)
        unit = "GB"
    else:
        displayed_size = float(size/MEM_TERABYTE_FACTOR)
        unit = "TB"
    str_list = [str(int(math.ceil(displayed_size))), unit]
    size_str = " ".join(str_list)
    return size_str

def get_os():
    '''
    获取操作系统信息
    '''
    os_name = None
    release_name = None
    if os.path.exists(OSPATH):
        with open(OSPATH, "r") as fsys:
            for line in fsys:
                if line.startswith("NAME"):
                    tmp = line
                    os_name = tmp.split('=')[1].replace('"', '').replace('\n', '')
                    #break
                elif line.startswith("PRETTY_NAME"):
                    tmp = line
                    release_name = tmp.split('=')[1].replace('"', '').replace('\n', '')
    else:
        if os.path.exists(RELEASEPATH):
            with open(RELEASEPATH, "r") as fsys:
                for line in fsys:
                    if line.startswith("DISTRIB_ID"):
                        tmp = line
                        os_name = tmp.split('=')[1].replace('"', '').replace('\n', '')
                    elif line.startswith("DISTRIB_RELEASE"):
                        tmp = line
                        release_name = tmp.split('=')[1].replace('"', '').replace('\n', '')
                        #break
    return (os_name, release_name)

def get_desktop():
    '''
    获取桌面环境信息
    '''
    #apt_cache = apt.Cache()
    #try:
    #    package = apt_cache["kylin-menu"]
    #except:
    #    package = None
    #if package is not None and package.is_installed:
    #    return "UKUI"
    return "UKUI"
    desktop_dict = {'ubuntu': 'Unity',
                    'ubuntu-2d': 'Unity 2D',
                    'gnome': 'GNOME Shell',
                    'gnome-classic': 'GNOME Classic',
                    'gnome-shell': 'GNOME Shell',
                    'gnome-fallback': 'GNOME Fallback',
                    'pantheon': 'elementary OS (Luna)',
                    'Lubutu': 'LXDE',
    }
    try:
        desktop_name = os.getenv('XDG_CURRENT_DESKTOP')
        if desktop_name is None:
            desktop_name = os.getenv('XDG_SESSION_DESKTOP')
        if desktop_name is None:
            desktop_name = "N/A"
        return desktop_name
    except Exception as e:
        desktop = os.getenv('DESKTOP_SESSION')
        if desktop in desktop_dict:
            return desktop_dict[desktop]
        else:
            if desktop:
                return 'Unknown (%s)' % desktop
            else:
                return 'Unknown'

def get_cpu():
    '''
    获取CPU信息
    '''
    cpu_info = None
    model_name = ''
    cpu_cores = 0
    # CPU
    machine = platform.machine()
    if machine == "aarch64":
        with open('/proc/cpuinfo') as f:
            for line in f:
                if line.strip():
                    if line.rstrip('\n').startswith('processor'):
                       cpu_cores += 1
    else:
        with open('/proc/cpuinfo') as f:
            for line in f:
                if line.strip():
                    if line.rstrip('\n').startswith('cpu cores'):
                       cpu_cores = int(line.rstrip('\n').split(':')[1].strip())
                       break
    with open('/proc/cpuinfo') as f:
        for line in f:
            if line.strip():
                if line.rstrip('\n').startswith('model name'):
                    model_name = line.rstrip('\n').split(':')[1].strip()
                    break
                elif line.rstrip('\n').startswith('Hardware'):
                    model_name = line.rstrip('\n').split(':')[1].strip()
                    break
    arch = " " + platform.architecture()[0]

    if cpu_cores == 0:
        cpu_info = model_name
    elif cpu_cores == 1:
        cpu_info = _("Single core ")  + model_name + arch#单核
    elif cpu_cores == 2:
        cpu_info = _("Double cores ")  + model_name + arch#双核
    elif cpu_cores == 3:
        cpu_info = _("Three cores ")  + model_name + arch#三核
    elif cpu_cores == 4:
        cpu_info = _("Four cores ")  + model_name + arch#四核
    else:
        cpu_info = str(cpu_cores) + _("cores ")  + model_name + arch
    return cpu_info

def page_memory():
    print("page_memory")
    MemSize = None
    # Memory
    with open('/proc/meminfo') as f:
        for line in f:
            if line.strip():
                if line.rstrip('\n').startswith('MemTotal'):
                    MemTotal = line.rstrip('\n').split(':')[1].strip()
                    MemTmp = MemTotal.split(' ')[0]
                    #MemSize = get_mem_human_read_capacity_size(int(MemTmp)*1024) #MemTmp = 3476412#8156252#7889972
                    #bit_len = int(MemTmp).bit_length() - 10 -10
                    #MemSize = str(2 ** bit_len) + " GB"
                    if MemTmp:
                        MemTmp = int(MemTmp)*1024
                        MemTmp = math.ceil(float(MemTmp) / 1024**3)
                        if MemTmp%2 != 0:
                            MemTmp = MemTmp + 1
                        MemSize = ("%1.f") % MemTmp + " GB"
                        break
    if MemSize is None:
        return "N/A"
    else:
        return str(MemSize)

#忽略单位为kB的内存数值
def dmi_memory():
    MemSize = ""
    mb_size = 0
    gb_size = 0
    hw = os.popen("/usr/sbin/dmidecode -t memory")
    memory = hw.read()
    hw.close()
    q = re.findall('Memory Device\n', memory)
    if q :
        memory = memory[memory.index("Memory Device\n") + len("Memory Device\n"):]
    else :
        memory = ''
    if memory:
        size_list = re.findall("[^ ]Size: (.*)", memory) #size 
        i = 0
        if size_list:
            for k in size_list :
                i += 1
                if size_list[i-1] not in [None, 'Unknown', 'None', '', 'No Module Installed']:
                    mem_size = size_list[i-1].split(' ')[0]
                    mem_unit = size_list[i-1].split(' ')[1]
                    if mem_unit == "MB":# 16384 MB  (16 GB)
                        tmp_size = int(mem_size) * 1024 * 1024#int(mem_size) * 1000 * 1000
                        if tmp_size < MEM_GIGABYTE_FACTOR:
                            mb_size += int(mem_size)
                        elif tmp_size < MEM_TERABYTE_FACTOR:
                            gb_size += int(("%1.f") % (float(tmp_size) / MEM_GIGABYTE_FACTOR))
                    elif mem_unit == "GB":
                        gb_size += int(mem_size)
                    elif mem_unit == "TB":
                        gb_size += int(mem_size) * 1024#gb_size += int(mem_size) *1000
    if gb_size == 0 and mb_size == 0:
        MemSize = "N/A"

    if gb_size > 0:
        MemSize = str(gb_size) + " GB"
    else:
        if mb_size > 0:
            MemSize = str(mb_size) + " MB"
        else:
            MemSize = "N/A"

    if MemSize == "N/A":
        MemSize = page_memory()

    return MemSize

def get_memory():
    '''
    获取内存信息
    '''
    MemSize = None
    is_efi = False
    #if not os.path.exists("/dev/mem"):
    #    is_efi = False
    #else:
    #    #有些FT2000+的机器，虽然不存在以下文件，
    #    if not os.path.exists("/sys/firmware/efi/systab") and not os.path.exists("/proc/efi/systab"):
    #        hw = os.popen("dmidecode")
    #        result = hw.read()
    #        hw.close()
    #        q = re.findall('No SMBIOS nor DMI entry point found', result)
    #        if q:
    #            is_efi = False
    #        else :
    #            is_efi = True
    #    else:
    #        is_efi = True
    #if is_efi:
    #    MemSize = dmi_memory()
    #else:
    #    MemSize = page_memory()
    MemSize = dmi_memory()
    return str(MemSize)

#removable 0: disk    1: usb
def is_disk_or_usb(filepath):
    filename = "/sys/block/" + filepath +  os.path.sep + 'removable'
    if os.path.exists(filename):
        try:
            fp = open(filename, "r")
            content = fp.read()
            content = content.strip('\n')
            fp.close()
            if int(content) == 0:
                return True
            else:#当设备为可插拔设备时，则通过检查设备信息中是否存在usb字段来过滤usb设备
                ds = os.popen("readlink -f /sys/block/%s/device" % filepath)
                content2 = ds.read()
                ds.close()
                if content2:
                    if re.findall('usb', content2):
                        return False
                return True
        except Exception as e:
            return False
    return False

def get_hardware():
    '''
    获取硬盘信息，支持多个硬盘，包括固态硬盘
    '''
    hd_list = []
    fpath = os.path.expanduser("/sys/block/")
    for filepath in os.listdir(fpath):
        if filepath.startswith('loop') or filepath.startswith('dm-') or filepath.startswith('ram') or filepath.startswith('sr') or filepath.startswith('.'):
            continue

        if (is_disk_or_usb(filepath) == False):
            continue

        filename = "/sys/block/" + filepath +  os.path.sep + 'size'
        if os.path.exists(filename):
            try:
                fp = open(filename, "r")
                content = fp.read()
                content = content.strip('\n')
                fp.close()
                if int(content) == 0:
                    continue
                else:
                    hd_sector_size = 512
                    #phyfilename = "/sys/block/" + filepath +  os.path.sep + 'queue/physical_block_size'
                    #phyfilename = "/sys/block/" + filepath +  os.path.sep + 'queue/logical_block_size'
                    #if os.path.exists(phyfilename):
                        #fp = open(phyfilename, "r")
                        #hd_sector_size = int(fp.read())
                        #fp.close()
                    hd_list.append(get_human_read_capacity_size(int(content) * hd_sector_size))
            except Exception as e:
                continue

    if len(hd_list) > 0:
        hd_info ="+".join(hd_list)
        return hd_info
    else:
        return "N/A"


INTERFACE = "com.kylin.computer"
PATH = "/"
HOME = os.path.expandvars('$HOME')


class Daemon(dbus.service.Object):
    def __init__ (self, mainloop):
        '''
        dbus进程初始化
        '''
        bus_name = dbus.service.BusName(INTERFACE, bus=dbus.SystemBus())#bus=dbus.SessionBus()
        dbus.service.Object.__init__(self, bus_name, PATH)
        self.mainloop = mainloop

    @dbus.service.method(INTERFACE, in_signature='', out_signature='ssssssss')
    def get_computer_info(self):
        '''
        获取系统信息
        '''
        (os_name, release_name) = get_os()
        kernel_name = platform.platform()
        #import socket, getpass, os, platform
        #user_home = os.path.expanduser('~')
        #user_name = getpass.getuser()
        #host_name = socket.gethostname()#platform.uname()[1]
        login_user = GLib.get_user_name()
        desktop_type = get_desktop()
        cpu = get_cpu()
        memory = get_memory()
        hardcapacity = get_hardware()
        return (os_name, release_name, kernel_name, login_user, desktop_type, cpu, memory, hardcapacity)
        
    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        '''
        dbus退出函数
        '''
        self.mainloop.quit()

    @dbus.service.method(INTERFACE, in_signature='', out_signature='s')
    def call_computer_daemon(self):
        '''
        一个dbus的测试函数
        '''
        return "My Computer Property"

if __name__ == '__main__':
    '''
    主函数   使用#echo $LANG或#locale查看当前系统语言环境
    '''
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    GObject.threads_init()
    #en_trans = gettext.translation('kylin-computer-viewer', locale_path, languages=['en_US'])
    #en_trans.install()
    #zh_trans = gettext.translation('kylin-computer-viewer', locale_path, languages=['zh_CN'])
    #zh_trans.install()
    mainloop = GObject.MainLoop()
    signal.signal(signal.SIGINT, lambda : mainloop.quit())
    Daemon(mainloop)
    mainloop.run()

